@@ -29,11 +29,10 @@
 # endif /* CAPABILITIES_NATIVE */
 #endif /* CAPABILITIES */
 
-#define USE_TNC /* for tnccs_manager */
-
 #include "daemon.h"
 
 #include <library.h>
+#include <plugins/plugin.h>
 #include <config/proposal.h>
 #include <kernel/kernel_handler.h>
 #include <processing/jobs/start_action_job.h>
@@ -106,6 +105,10 @@ static void destroy(private_daemon_t *this)
 	{
 		this->public.ike_sa_manager->flush(this->public.ike_sa_manager);
 	}
+	if (this->public.traps)
+	{
+		this->public.traps->flush(this->public.traps);
+	}
 	DESTROY_IF(this->public.receiver);
 	DESTROY_IF(this->public.sender);
 	/* unload plugins to release threads */
@@ -119,8 +122,6 @@ static void destroy(private_daemon_t *this)
 	DESTROY_IF(this->public.ike_sa_manager);
 	DESTROY_IF(this->public.controller);
 	DESTROY_IF(this->public.eap);
-	DESTROY_IF(this->public.sim);
-	DESTROY_IF(this->public.tnccs);
 #ifdef ME
 	DESTROY_IF(this->public.connect_manager);
 	DESTROY_IF(this->public.mediation_manager);
@@ -210,7 +211,7 @@ static void print_plugins()
 
 	buf[0] = '\0';
 	enumerator = lib->plugins->create_plugin_enumerator(lib->plugins);
-	while (len < sizeof(buf) && enumerator->enumerate(enumerator, &plugin))
+	while (len < sizeof(buf) && enumerator->enumerate(enumerator, &plugin, NULL))
 	{
 		len += snprintf(&buf[len], sizeof(buf)-len, "%s ",
 						plugin->get_name(plugin));
@@ -290,8 +291,6 @@ private_daemon_t *daemon_create()
 	charon = &this->public;
 	this->public.controller = controller_create();
 	this->public.eap = eap_manager_create();
-	this->public.sim = sim_manager_create();
-	this->public.tnccs = tnccs_manager_create();
 	this->public.backends = backend_manager_create();
 	this->public.socket = socket_manager_create();
 	this->public.traps = trap_manager_create();
