@@ -281,6 +281,10 @@ static void process_ike_update(private_ha_dispatcher_t *this,
 				set_extension(ike_sa, value.u32, EXT_NATT);
 				set_extension(ike_sa, value.u32, EXT_MOBIKE);
 				set_extension(ike_sa, value.u32, EXT_HASH_AND_URL);
+				set_extension(ike_sa, value.u32, EXT_MULTIPLE_AUTH);
+				set_extension(ike_sa, value.u32, EXT_STRONGSWAN);
+				set_extension(ike_sa, value.u32, EXT_EAP_ONLY_AUTHENTICATION);
+				set_extension(ike_sa, value.u32, EXT_MS_WINDOWS);
 				break;
 			case HA_CONDITIONS:
 				set_condition(ike_sa, value.u32, COND_NAT_ANY);
@@ -290,6 +294,7 @@ static void process_ike_update(private_ha_dispatcher_t *this,
 				set_condition(ike_sa, value.u32, COND_EAP_AUTHENTICATED);
 				set_condition(ike_sa, value.u32, COND_CERTREQ_SEEN);
 				set_condition(ike_sa, value.u32, COND_ORIGINAL_INITIATOR);
+				set_condition(ike_sa, value.u32, COND_STALE);
 				break;
 			default:
 				break;
