@@ -13,10 +13,11 @@
  * for more details.
  */
 
-#include "tnc_imc.h"
+#include <tnc/tnc.h>
+#include <tnc/imc/imc_manager.h>
+#include <tnc/tnccs/tnccs_manager.h>
 
 #include <debug.h>
-#include <daemon.h>
 
 #define TNC_IMVID_ANY	0xffff
 
@@ -28,14 +29,14 @@ TNC_Result TNC_TNCC_ReportMessageTypes(TNC_IMCID imc_id,
 									   TNC_MessageTypeList supported_types,
 									   TNC_UInt32 type_count)
 {
-	if (!charon->imcs->is_registered(charon->imcs, imc_id))
+	if (!tnc->imcs->is_registered(tnc->imcs, imc_id))
 	{
 		DBG1(DBG_TNC, "ignoring ReportMessageTypes() from unregistered IMC %u",
 					   imc_id);
 		return TNC_RESULT_INVALID_PARAMETER;
 	}
-	return charon->imcs->set_message_types(charon->imcs, imc_id,
-										   supported_types, type_count);
+	return tnc->imcs->set_message_types(tnc->imcs, imc_id, supported_types,
+										type_count);
 }
 
 /**
@@ -45,14 +46,14 @@ TNC_Result TNC_TNCC_RequestHandshakeRetry(TNC_IMCID imc_id,
 										  TNC_ConnectionID connection_id,
 										  TNC_RetryReason reason)
 {
-	if (!charon->imcs->is_registered(charon->imcs, imc_id))
+	if (!tnc->imcs->is_registered(tnc->imcs, imc_id))
 	{
 		DBG1(DBG_TNC, "ignoring RequestHandshakeRetry() from unregistered IMC %u",
 					   imc_id);
 		return TNC_RESULT_INVALID_PARAMETER;
 	}
-	return charon->tnccs->request_handshake_retry(charon->tnccs, TRUE, imc_id,
-												  connection_id, reason);
+	return tnc->tnccs->request_handshake_retry(tnc->tnccs, TRUE, imc_id,
+											   connection_id, reason);
 }
 
 /**
@@ -64,14 +65,14 @@ TNC_Result TNC_TNCC_SendMessage(TNC_IMCID imc_id,
 								TNC_UInt32 msg_len,
 								TNC_MessageType msg_type)
 {
-	if (!charon->imcs->is_registered(charon->imcs, imc_id))
+	if (!tnc->imcs->is_registered(tnc->imcs, imc_id))
 	{
 		DBG1(DBG_TNC, "ignoring SendMessage() from unregistered IMC %u",
 					   imc_id);
 		return TNC_RESULT_INVALID_PARAMETER;
 	}
-	return charon->tnccs->send_message(charon->tnccs, imc_id, TNC_IMVID_ANY,
-									   connection_id, msg, msg_len, msg_type);
+	return tnc->tnccs->send_message(tnc->tnccs, imc_id, TNC_IMVID_ANY,
+									connection_id, msg, msg_len, msg_type);
 }
 
 /**
