@@ -14,14 +14,12 @@
  */
 
 #include "tnc_imc_manager.h"
-
-#include <tnc/imc/imc_manager.h>
+#include "tnc_imc.h"
 
 #include <tncifimc.h>
 
-#include <debug.h>
-#include <library.h>
 #include <utils/linked_list.h>
+#include <debug.h>
 
 typedef struct private_tnc_imc_manager_t private_tnc_imc_manager_t;
 
@@ -95,6 +93,33 @@ METHOD(imc_manager_t, remove_, imc_t*,
 	return removed_imc;
 }
 
+METHOD(imc_manager_t, load, bool,
+	private_tnc_imc_manager_t *this, char *name, char *path)
+{
+	imc_t *imc;
+
+	imc = tnc_imc_create(name, path);
+	if (!imc)
+	{
+		free(name);
+		free(path);
+		return FALSE;
+	}
+	if (!add(this, imc))
+	{
+		if (imc->terminate &&
+			imc->terminate(imc->get_id(imc)) != TNC_RESULT_SUCCESS)
+		{
+			DBG1(DBG_TNC, "IMC \"%s\" not terminated successfully",
+						   imc->get_name(imc));
+		}
+		imc->destroy(imc);
+		return FALSE;
+	}
+	DBG1(DBG_TNC, "IMC %u \"%s\" loaded from '%s'", imc->get_id(imc), name, path);
+	return TRUE;
+}
+
 METHOD(imc_manager_t, is_registered, bool,
 	private_tnc_imc_manager_t *this, TNC_IMCID id)
 {
@@ -252,6 +277,7 @@ imc_manager_t* tnc_imc_manager_create(void)
 		.public = {
 			.add = _add,
 			.remove = _remove_, /* avoid name conflict with stdio.h */
+			.load = _load,
 			.is_registered = _is_registered,
 			.get_preferred_language = _get_preferred_language,
 			.notify_connection_change = _notify_connection_change,
