@@ -13,10 +13,11 @@
  * for more details.
  */
 
-#include "tnc_imv.h"
+#include <tnc/tnc.h>
+#include <tnc/imv/imv_manager.h>
+#include <tnc/tnccs/tnccs_manager.h>
 
 #include <debug.h>
-#include <daemon.h>
 
 #define TNC_IMCID_ANY	0xffff
 
@@ -28,14 +29,14 @@ TNC_Result TNC_TNCS_ReportMessageTypes(TNC_IMVID imv_id,
 									   TNC_MessageTypeList supported_types,
 									   TNC_UInt32 type_count)
 {
-	if (!charon->imvs->is_registered(charon->imvs, imv_id))
+	if (!tnc->imvs->is_registered(tnc->imvs, imv_id))
 	{
 		DBG1(DBG_TNC, "ignoring ReportMessageTypes() from unregistered IMV %u",
 					   imv_id);
 		return TNC_RESULT_INVALID_PARAMETER;
 	}
-	return charon->imvs->set_message_types(charon->imvs, imv_id,
-										   supported_types, type_count);
+	return tnc->imvs->set_message_types(tnc->imvs, imv_id, supported_types,
+										type_count);
 }
 
 /**
@@ -45,14 +46,14 @@ TNC_Result TNC_TNCS_RequestHandshakeRetry(TNC_IMVID imv_id,
 										  TNC_ConnectionID connection_id,
 										  TNC_RetryReason reason)
 {
-	if (!charon->imvs->is_registered(charon->imvs, imv_id))
+	if (!tnc->imvs->is_registered(tnc->imvs, imv_id))
 	{
 		DBG1(DBG_TNC, "ignoring RequestHandshakeRetry() from unregistered IMV %u",
 					   imv_id);
 		return TNC_RESULT_INVALID_PARAMETER;
 	}
-	return charon->tnccs->request_handshake_retry(charon->tnccs, FALSE, imv_id,
-												  connection_id, reason);
+	return tnc->tnccs->request_handshake_retry(tnc->tnccs, FALSE, imv_id,
+											   connection_id, reason);
 }
 
 /**
@@ -64,14 +65,14 @@ TNC_Result TNC_TNCS_SendMessage(TNC_IMVID imv_id,
 								TNC_UInt32 msg_len,
 								TNC_MessageType msg_type)
 {
-	if (!charon->imvs->is_registered(charon->imvs, imv_id))
+	if (!tnc->imvs->is_registered(tnc->imvs, imv_id))
 	{
 		DBG1(DBG_TNC, "ignoring SendMessage() from unregistered IMV %u",
 					   imv_id);
 		return TNC_RESULT_INVALID_PARAMETER;
 	}
-	return charon->tnccs->send_message(charon->tnccs, TNC_IMCID_ANY, imv_id,
-									   connection_id, msg, msg_len, msg_type);
+	return tnc->tnccs->send_message(tnc->tnccs, TNC_IMCID_ANY, imv_id,
+									connection_id, msg, msg_len, msg_type);
 }
 
 /**
@@ -83,14 +84,14 @@ TNC_Result TNC_TNCS_ProvideRecommendation(TNC_IMVID imv_id,
 								TNC_IMV_Action_Recommendation recommendation,
 								TNC_IMV_Evaluation_Result evaluation)
 {
-	if (!charon->imvs->is_registered(charon->imvs, imv_id))
+	if (!tnc->imvs->is_registered(tnc->imvs, imv_id))
 	{
 		DBG1(DBG_TNC, "ignoring ProvideRecommendation() from unregistered IMV %u",
 					   imv_id);
 		return TNC_RESULT_INVALID_PARAMETER;
 	}
-	return charon->tnccs->provide_recommendation(charon->tnccs, imv_id,
-							connection_id, recommendation, evaluation);
+	return tnc->tnccs->provide_recommendation(tnc->tnccs, imv_id, connection_id,
+											  recommendation, evaluation);
 }
 
 /**
@@ -104,13 +105,13 @@ TNC_Result TNC_TNCS_GetAttribute(TNC_IMVID imv_id,
 								 TNC_BufferReference buffer,
 								 TNC_UInt32 *out_value_len)
 {
-	if (!charon->imvs->is_registered(charon->imvs, imv_id))
+	if (!tnc->imvs->is_registered(tnc->imvs, imv_id))
 	{
 		DBG1(DBG_TNC, "ignoring GetAttribute() from unregistered IMV %u",
 					   imv_id);
 		return TNC_RESULT_INVALID_PARAMETER;
 	}
-	return charon->tnccs->get_attribute(charon->tnccs, imv_id, connection_id,
+	return tnc->tnccs->get_attribute(tnc->tnccs, imv_id, connection_id,
 							attribute_id, buffer_len, buffer, out_value_len);
 }
 
@@ -124,14 +125,14 @@ TNC_Result TNC_TNCS_SetAttribute(TNC_IMVID imv_id,
 								 TNC_UInt32 buffer_len,
 								 TNC_BufferReference buffer)
 {
-	if (!charon->imvs->is_registered(charon->imvs, imv_id))
+	if (!tnc->imvs->is_registered(tnc->imvs, imv_id))
 	{
 		DBG1(DBG_TNC, "ignoring SetAttribute() from unregistered IMV %u",
 					   imv_id);
 		return TNC_RESULT_INVALID_PARAMETER;
 	}
-	return charon->tnccs->set_attribute(charon->tnccs, imv_id, connection_id,
-										attribute_id, buffer_len, buffer);
+	return tnc->tnccs->set_attribute(tnc->tnccs, imv_id, connection_id,
+									 attribute_id, buffer_len, buffer);
 }
 
 /**
