@@ -14,15 +14,22 @@
  */
 
 #include "tnc_imv_manager.h"
+#include "tnc_imv.h"
 #include "tnc_imv_recommendations.h"
 
-#include <tnc/imv/imv_manager.h>
-
 #include <tncifimv.h>
 #include <tncif_names.h>
 
-#include <debug.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/mman.h>
+#include <unistd.h>
+#include <errno.h>
+#include <fcntl.h>
+
 #include <daemon.h>
+#include <utils/lexparser.h>
+#include <debug.h>
 #include <threading/mutex.h>
 
 typedef struct private_tnc_imv_manager_t private_tnc_imv_manager_t;
@@ -103,6 +110,33 @@ METHOD(imv_manager_t, remove_, imv_t*,
 	return removed_imv;
 }
 
+METHOD(imv_manager_t, load, bool,
+	private_tnc_imv_manager_t *this, char *name, char *path)
+{
+	imv_t *imv;
+
+	imv = tnc_imv_create(name, path);
+	if (!imv)
+	{
+		free(name);
+		free(path);
+		return FALSE;
+	}
+	if (!add(this, imv))
+	{
+		if (imv->terminate &&
+			imv->terminate(imv->get_id(imv)) != TNC_RESULT_SUCCESS)
+		{
+			DBG1(DBG_TNC, "IMV \"%s\" not terminated successfully",
+						   imv->get_name(imv));
+		}
+		imv->destroy(imv);
+		return FALSE;
+	}
+	DBG1(DBG_TNC, "IMV %u \"%s\" loaded from '%s'", imv->get_id(imv), name, path);
+	return TRUE;
+}
+
 METHOD(imv_manager_t, is_registered, bool,
 	private_tnc_imv_manager_t *this, TNC_IMVID id)
 {
@@ -291,6 +325,7 @@ METHOD(imv_manager_t, batch_ending, void,
 	enumerator->destroy(enumerator);
 }
 
+
 METHOD(imv_manager_t, destroy, void,
 	private_tnc_imv_manager_t *this)
 {
@@ -322,6 +357,7 @@ imv_manager_t* tnc_imv_manager_create(void)
 		.public = {
 			.add = _add,
 			.remove = _remove_, /* avoid name conflict with stdio.h */
+			.load = _load,
 			.is_registered = _is_registered,
 			.get_recommendation_policy = _get_recommendation_policy,
 			.create_recommendations = _create_recommendations,
@@ -336,6 +372,7 @@ imv_manager_t* tnc_imv_manager_create(void)
 		.imvs = linked_list_create(),
 		.next_imv_id = 1,
 	);
+
 	policy = enum_from_name(recommendation_policy_names,
 				lib->settings->get_str(lib->settings,
 					"charon.plugins.tnc-imv.recommendation_policy", "default"));
